/*-------------------------------------------------------------------------*\

  FILE..: MODULATE.CPP
  AUTHOR: Peter Wintulich
  DATE..: 9 MAY 2005

  FM data encoding functions for caller ID functions.
  
\*-------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\

         Voicetronix Voice Processing Board (VPB) Software

         Copyright (C) 1999-2001 Voicetronix www.voicetronix.com.au

         This library is free software; you can redistribute it and/or
         modify it under the terms of the GNU Lesser General Public
         License as published by the Free Software Foundation; either
         version 2.1 of the License, or (at your option) any later version.

         This library is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
         Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
         License along with this library; if not, write to the Free Software
         Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	 USA

\*---------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*\

				   INCLUDES

\*-------------------------------------------------------------------------*/


#include <math.h>
#include "vpbapi.h"
#include "modulate.h"

/*-----------------------------------------------------------------------*\

	FUNCTION: byte_encode() 
	AUTHOR..: Peter Wintulich
	DATE....: Nov 2002
 
	Encode byte from DLPB into wave buffer *BUFF using bit_encode()
 
\*-----------------------------------------------------------------------*/

int byte_encode(char dlpb, int *scount, short *buff, double *stime, int *bt, int country)
{	//	dlpb		Datalink Layer Packet Byte to encode
	//	*scount		Pointer for wave buffer current sample
	//	*buff		Pointer to head of wave buffer
	//	*stime		SINE time
	//	*bt		Bit Time

	int	i,p;
	int	sc;		// sample count per bit
	char	byte;		// temp char to shift bits out with

        p=*scount;
	bit_encode(0, &sc, &buff[*scount], stime, bt, country);	// Start Bit
	*scount +=sc;
	
	byte= dlpb;				// data bits 0-7
	for (i=0; i<8 ; i++)
	{	
		bit_encode((byte & 1? 1:0), &sc, &buff[*scount], stime, bt, country);
		*scount +=sc;
		byte = byte >> 1;
	}
	
	bit_encode(1, &sc, &buff[*scount], stime, bt, country);	// Stop Bit
	*scount +=sc;
	
	return 0;	
}


/*-----------------------------------------------------------------------*\

	FUNCTION: bit_encode()
	AUTHOR..: Peter Wintulich
	DATE....: Nov 2002

        'Acc' & 'bt' need to be preserved by the cid composer (per
        thread) to maintain the phase relation ship and prevent bit
        width from accumilating any error, therefor causing a farming
        error.
 
        LAMP & HAMP are amplitudes for the sine generator,
        LAMP for mark, HAMP for space

\*-----------------------------------------------------------------------*/

int bit_encode(int bit, int *samples_count, short *samples, double *Acc, int *bt, int country)
{
	// bit       	   Logic of bit to compose
	// *samples_count  returns number of samples
	// *samples        array of short samples
	// *Acc		   Accumilation of sine Radians
	// *bt		   Bit time refrence
	// country	   modulation standard

	int	x;
	int samples_bt[3] = {7,6,7};	// Number of sample times per bit
	

	for(x=0; x < samples_bt[(*bt)%3]; x++)	
	{
	    switch(country)	// select tone set to use.
	    {
		case VPB_COUNTRY_JP:   
		case VPB_COUNTRY_UK:   
		    if(bit == 1)        //1300Hz
                    {
		        *Acc= *Acc + 1.021017612L;
		        samples[x] = (short) (LAMP * sin(*Acc));
		    }
                    else                //2100Hz
		    {
		        *Acc= *Acc + 1.649336143L;
		        samples[x] = (short) (HAMP * sin( *Acc));
		    }
		    break;
					
		case VPB_COUNTRY_USA:
	    	case VPB_COUNTRY_AU:	
	        default:	    
		    if(bit == 1) 	//1200Hz
		    {	    
			*Acc= *Acc + 0.942477769L;
			samples[x] = (short) (LAMP * sin( *Acc));
		    }	
		    else		//2200Hz
		    {
			*Acc= *Acc + 1.72787596L;	    
			samples[x] = (short) (HAMP * sin( *Acc));
		    }	
		    break;
	    }	    
	}
	*samples_count = samples_bt[*bt];
	(*bt)=((*bt)+1)%3;	// advance bit time
	
	return	0;
}

