/*---------------------------------------------------------------------------*\

	FILE....: cidgtest.cpp
	TYPE....: C Program
	AUTHOR..: Peter Wintulich
	DATE....: 12/11/02

	Test program for OpenSwitch12 Caller ID transmit
	

	Compile: gcc cidgtest.cpp cidg.cpp -o cidgtest -lvpb -lm -pthread -Wall

\---------------------------------------------------------------------------*/
	
#include <assert.h>
#include <time.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../../src/vpbapi.h"
#include "../../src/verbose.h"
#include "../kbhit.h"


int DID_out(int h, char *num, char *text, char *argv[]);
int ctwaitforevent(int h, int event_type, int timeout_ms);

int WINAPI vpb_cid_jp_set(VPB_CID_JP *cid_struct, int type, void *value);
int WINAPI vpb_cid_jp_compose_dlp(VPB_CID_JP *cid_struct);
int WINAPI vpb_cid_jp_compose_wav(VPB_CID_JP *cid_struct, short *buff_wav, int *buff_cnt);


static int finito=0;

/*-----------------------------------------------------------------------\
 
 
 
 
\-----------------------------------------------------------------------*/
int main(int argc, char *argv[]) 
{
	int	h;			// vpb channel handle
	int     ch_num;			// channel #
	short   buf[VPB_CID_MAX_BUF];	// wave holding buffer
	int	buff_count=0;		// active word count in buf
	int	x,n;
	char	VPB_DLP[VPB_CID_MAX_DLP];	// Datalink Layer 
	int 	pg,lamp,hamp;	
	char	*file=NULL;		// pointer to output filename
    	void 	*ws;			// file handle
	char	number[50];
	char	txtmessage[50];

	//verbose(1);
	vpb_seterrormode(VPB_DEVELOPMENT);
	
	if (argc == 1) {
	    printf("Usage: ./cidgtest file.wav Ch[1..] P[gain]\n");
	    printf("Usage: ./cidgtest 1 Ch[1..] [P[gain]] [F[fileout.wav]]\n");
	    printf("                  [m[message]] [n[number]]\n");
		exit(0);
	}
	
	strcpy(number,"8451 5108");
	strcpy(txtmessage,"Voicetronix");
	
	ch_num = atoi(argv[2]);
	h = vpb_open(1, ch_num);

	char num[]="1234567890";
	char text[]="abcdefghij";
	DID_out(h, num, text, argv);


#ifdef FRED
	// Collect optional parameters
	if (argc >3)
	{	
  	    for(x=3; x<argc ; x++)
	       switch(*(argv[x]))
	       {
		   case 'h':
		   case 'H': 	 
	       		hamp=atoi(argv[x]+1);	       
	       		printf("High F amp=%d\n",hamp);	       
			printf("not implimented\n");
			break;	
		   case 'l':
		   case 'L': 	 
	       		lamp=atoi(argv[x]+1);	       
	       		printf("Low F amp=%d\n",lamp);
	 		printf("not inplemented\n");		
			break;
		   case 'p':
		   case 'P': 	 
	       		pg=atoi(argv[x]+1);	       
	       		printf("Play Gain=%d\n",pg);	       
			vpb_play_set_gain(h,pg);
			break;
		   case 'f':
		   case 'F': 	 
	       		file=(argv[x])+1;	       
	       		printf("Wave File=%s\n",file);	       
			break;
		   case 'm':
		   case 'M': 	 
	       		strcpy(txtmessage,(argv[x])+1);	       
	       		printf("Message=%s\n",txtmessage);	       
			break;
		   case 'n':
		   case 'N': 	 
	       		strcpy(number,(argv[x])+1);	       
	       		printf("Number=%s\n",number);	       
			break;
	       }	
		
	}
	
	//----------------Message creation-----------------------------	
	VPB_CID	message;			// create VPB_CID structure
	vpb_cid_set(&message,VPB_CID_EMPTY,NULL);	// Initalize fields

	vpb_cid_set(&message,VPB_CID_CALLING_LINE_DN,(void *)number);
	vpb_cid_set(&message,VPB_CID_CALLER_NAME,(void *)txtmessage);
//	vpb_cid_set(&message,VPB_CID_CALLING_LINE_DN,(void *)"84155108");
//	vpb_cid_set(&message,VPB_CID_CALLER_NAME,(void *)"Voicetronix");
	n= vpb_cid_compose_dlp(&message, VPB_DLP);

	vpb_cid_compose_wav(VPB_DLP, n, buf, &buff_count);

	printf("buff_count = %d \n",buff_count);

	x=0;

	//	Print hex dump of DLP to screen
	//	for(x=0;x<buff_count;x++)
	//	printf("%x ,%d \n",x,buf[x]);
	
	//-----------Write composed wave to file--------------------
			
    	if(file!=NULL) 
    	{	    
		vpb_wave_open_write(&ws, file, VPB_LINEAR);
		vpb_wave_write(ws,(char*)buf,buff_count*2);
		vpb_wave_close_write(ws);
    	}
	//-----------Send First ring--------------------------------	
	vpb_sleep(1000);		// wait for card to initalize
	vpb_user_ring_station_sync(h,VPB_RING_STATION_ON);
	vpb_sleep(500);
	vpb_user_ring_station_sync(h,VPB_RING_STATION_OFF);
	vpb_sleep(600);
	//-----------Play file or buffer---------------------------- 
	if(*(argv[1]) == '1')
    	{	    
		printf("Sending composed CID\n");
	        vpb_play_buf_start(h, VPB_LINEAR);
	        vpb_play_buf_sync(h, (char*)buf, buff_count*2);
		vpb_sleep(50);
		vpb_play_buf_finish(h);
    	}	
    	else
    	{
		printf("Sending FILE\n");    
		vpb_play_file_sync(h,argv[1]);
    	}	
	//-----------Send second ring------------------------------
	vpb_sleep(400);
	vpb_user_ring_station_sync(h,VPB_RING_STATION_ON);
	vpb_sleep(800);
	vpb_user_ring_station_sync(h,VPB_RING_STATION_OFF);
#endif	
	vpb_close(h);	

	return 0;
}


/*-----------------------------------------------------------------------\

\-----------------------------------------------------------------------*/
int DID_out(int h, char *num, char *text, char *argv[])
{
	VPB_CID_JP did;
	//VPB_EVENT	ev;
	int	ev;
	int	x;	// 2x Number of cycles to fast ring
	short   *buf;	// wave holding buffer
	int	buff_count=0;		// active word count in buf

	buf = (short*)malloc(20000);

	vpb_cid_jp_set(&did, VPB_CID_EMPTY, NULL);	// Initalize fields
	vpb_cid_jp_set(&did, VPB_DID_OTN, (void *)num);
	vpb_cid_jp_set(&did, VPB_DID_CND,(void *)text);
	vpb_cid_jp_compose_dlp(&did);
	vpb_cid_jp_compose_wav(&did, buf, &buff_count);

	x=12;		// 6 seconds of fast ring 500ms/500ms on/off
	do{
		x--;
		if(x & 1)
			vpb_user_ring_station_sync(h,VPB_RING_STATION_ON);
		else
			vpb_user_ring_station_sync(h,VPB_RING_STATION_OFF);
		ev = ctwaitforevent(h, VPB_STATION_OFFHOOK, 500);
		if(ev == VPB_STATION_OFFHOOK)
		{
			if(x & 1)	// if still ringing shut it off
				vpb_user_ring_station_sync(h,VPB_RING_STATION_OFF);
			x=-1;	// flag that we got 'VPB_STATION_OFFHOOK' event
		}
	}while(x>0);
	if(x == -1)	// We got off hook from fast ring so..
	{
	    vpb_sleep(100);
	    if(*(argv[1]) == '1') // send composed buffer
	    {	    
		printf("Sending composed CID\n");
	        vpb_play_buf_start(h, VPB_LINEAR);
	        vpb_play_buf_sync(h, (char*)buf, buff_count*2);
		vpb_sleep(50);
		vpb_play_buf_finish(h);
	    }	
	    else	// or send wave file
	    {
		printf("Sending FILE\n");    
		vpb_play_file_sync(h,argv[1]);
	    }	
	    // Wait for 7 seconds, if cid is collected then 'VPB_STATION_ONHOOK'
	    // will arrive much sooner
	    ev = ctwaitforevent(h, VPB_STATION_ONHOOK, 7000);
	    if(ev == VPB_TIMEREXP)
		x= -2;	// telephone did not recignize DID TX 
	}
	free(buf);
	// if X == 0 then no trigger from STATION for DID TX
	// if X == -1 then CID was collected 
	// Ready for normal ring process
	return x;
}

int ctwaitforevent(int h, int event_type, int timeout_ms) {
	char      s[VPB_MAX_STR];
	int       ret, state;
	VPB_EVENT e;   
	void      *timer;
	short	  volts;

	vpb_timer_open(&timer, h, 0, timeout_ms);
	if (timeout_ms != 0)
		vpb_timer_start(timer);

	state = 1;
	while(state && !finito) {
		ret = vpb_get_event_ch_async(h, &e);

		if (ret == VPB_OK) {
			vpb_translate_event(&e, s);
			printf("%s",s);

			if (e.type == event_type) {
				state = 0;
			}
			if (e.type == VPB_TIMEREXP) {
				state = 0;
			}
			if(e.type == VPB_STATION_OFFHOOK) {
				state = 0;
			}
		}
		else
			vpb_sleep(100);
	}
	  
	vpb_timer_close(timer);	
	if (state == 0)
		return e.type;
	else
		return -1;
}
